package com.dgtly.sync.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dgtly.common.exception.BusinessException;
import com.dgtly.common.utils.ShiroSaltUtil;
import com.dgtly.common.utils.UserIdentityUtil;
import com.dgtly.common.utils.http.HttpUtils;
import com.dgtly.common.utils.security.EncryptPassWordClass;
import com.dgtly.sync.domain.Customers;
import com.dgtly.sync.domain.SyncFailDetail;
import com.dgtly.sync.domain.SyncLog;
import com.dgtly.sync.mapper.CustomersMapper;
import com.dgtly.sync.mapper.SyncFailDetailMapper;
import com.dgtly.sync.utils.CustomerAccessTokenUtil;
import com.dgtly.sync.utils.CustomerAccessTokenUtil.UrlType;
import com.dgtly.system.domain.AssRelcustomerinfo;
import com.dgtly.system.domain.SysUser;
import com.dgtly.system.domain.SysUserExt;
import com.dgtly.system.mapper.SysUserExtMapper;
import com.dgtly.system.mapper.SysUserMapper;
import com.dgtly.system.service.IAssRelcustomerinfoService;
import com.dgtly.system.service.ISysConfigService;
import com.dgtly.system.service.impl.SysUserServiceImpl;
import com.dgtly.wxportal.mapper.WxQyUserMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Component
public class AnalysisDiyCustomerComponent {

    private static final Logger log = LoggerFactory.getLogger(AnalysisDiyCustomerComponent.class);


    @Autowired
    private CustomerAccessTokenUtil customerAccessTokenUtil;

    @Autowired
    private CustomersMapper customersMapper;
    @Autowired
    private SysUserServiceImpl sysUserService;
    @Autowired
    private SysUserExtMapper sysUserExtMapper;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private WxQyUserMapper wxQyUserMapper;

    @Autowired
    private ISysConfigService configService;
    @Autowired
    private ISyncLogService syncLogService;
    @Autowired
    private SyncFailDetailMapper syncFailDetailMapper;
    @Autowired
    private IAssRelcustomerinfoService assRelcustomerinfoService;
    private Long deptId;
    //老板角色
    private Long[] roleIds;
    //财务
    private Long[] FinanceRoleIds;
    //仓管
    private Long[] HouseRoleIds;
    private Long[] roleGuideIds;
    private Long[] roleCwIds;
    private Long[] roleCwDiyIds;
    private Long[] roleFXDiyIds;  //3.4接口，同步分销用户
    private Long[] roleStoreIds; //3.6门店经销商
    private Long[] roleBusiness;//7.5厨卫业务人员角色
    private Long[] roleXls;//13.1新零售经销商角色
    /**
     * 获取并解析DIY经销商用户
     * 因为经销商用户并不涉及用信息更新等方面 所以只插入不更新
     * @return
     */
    //@Override
    public SyncLog analyDiyCustomer() {
        if(deptId==null){
            deptId = Long.parseLong(configService.selectConfigByKey("libang.customer.deptid"));
        }
        //老板角色
        if(roleIds==null){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.customer.roleid"));
            roleIds = new Long[]{roleId};
        }
        //财务
        if(FinanceRoleIds==null){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.customerFinance.roleid"));
            FinanceRoleIds = new Long[]{roleId};
        }
        //仓管
        if(HouseRoleIds==null){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.customerHouse.roleid"));
            HouseRoleIds = new Long[]{roleId};
        }
        if(roleXls ==null){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.customerXls.roleid"));
            roleXls = new Long[]{roleId};
        }

        if (roleGuideIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.guide.roleid"));
            roleGuideIds = new Long[]{roleId};
        }
        if (roleCwIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.roleid"));
            roleCwIds = new Long[]{roleId};
        }
        if (roleCwDiyIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.roleid"));
            Long roleDiyId = Long.parseLong(configService.selectConfigByKey("libang.guide.roleid"));
            roleCwDiyIds = new Long[]{roleId, roleDiyId};
        }
        if (roleFXDiyIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.fx.roleid"));
            roleFXDiyIds = new Long[]{roleId};
        }
        if (roleStoreIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.storeCustomer.roleid"));
            roleStoreIds = new Long[]{roleId};
        }


        List<SyncFailDetail> failDetails = new ArrayList<>();
        SyncLog syncLog = new SyncLog();
        syncLog.setStartTime(new Date());
        String packageId = "diycustomer"+syncLog.getStartTime().getTime();
        syncLog.setPackageId(packageId);
        long total = 0;
        long success = 0;
        Set<String> loginNameSet = sysUserService.selectAllUserLoginName();
        Set<String> diyNameSet = sysUserService.selectDiyUserLoginName();


        //导购经销商
        String guideJson ="";
        try{
            guideJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.STOREGUIDE,null));
        }catch (Exception e){
            log.error("获取经销商导购接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }
        Map<String,Customers> codeMap = customersMapper.selectCustomerMapByCode();
        JSONObject gjson = JSONObject.parseObject(guideJson);
        int code = gjson.getInteger("code");
        if(code==0){
            JSONArray STOREGUIDE = new JSONArray();
            STOREGUIDE.addAll(gjson.getJSONArray("userlist"));
            success+=buildSysUser(STOREGUIDE,loginNameSet,diyNameSet,0,failDetails,packageId,codeMap);
            total+=gjson.getJSONArray("userlist").size();
        }

        //经销商负责人
        String managerJson ="";
        try{
            managerJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.MANAGER,null));
        }catch (Exception e){
            log.error("获取经销商经理接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject mjson = JSONObject.parseObject(managerJson);


        Map<String,Customers> chainsMap = customersMapper.selectCustomerMapByChainsCode();
         code = mjson.getInteger("code");
        if(code==0){
            JSONArray MANAGER = new JSONArray();
            MANAGER.addAll(mjson.getJSONArray("userlist"));
           success+=buildSysUser(MANAGER,loginNameSet,diyNameSet,1,failDetails,packageId,chainsMap);
            total+=mjson.getJSONArray("userlist").size();
        }





        // 3.4、获取经销商分销业务员人员列表(分页)开始： ===================>
        log.info("3.4、获取经销商分销业务员人员列表(分页)开始： ===================>");
        String fxResellerJson ="";
        try{
            fxResellerJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.RESELLER,1+""));
        }catch (Exception e){
            log.error("获取经销商分销业务员人员列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject fxJson = JSONObject.parseObject(fxResellerJson);
        Integer fxCount = fxJson.getInteger("count");
        JSONArray jsonArray = new JSONArray();
        if(fxCount>100){
            int co = 0;
            if (fxCount%100 != 0){
                co = (fxCount/100) +2;
            }else{
                co = (fxCount/100) +1;
            }
            for (int i = 1; i < co; i++) {
                try {
                    fxJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.RESELLER,i+"")));
                } catch (Exception e) {
                    log.error("循环获取经销商财务接口错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = fxJson.getInteger("code");
                if(code==0){
                    jsonArray.addAll( fxJson.getJSONArray("userlist"));
                }
            }
            success+=buildSysUserFx(jsonArray,loginNameSet,6,failDetails,packageId,chainsMap);
            total+=jsonArray.size();

        }else{
            code = fxJson.getInteger("code");
            if(code==0){
                jsonArray.addAll( fxJson.getJSONArray("userlist"));
                success+=buildSysUserFx(jsonArray,loginNameSet,6,failDetails,packageId,chainsMap);
                total+=jsonArray.size();
            }
        }
        log.info("3.4、获取经销商分销业务员人员列表(分页)结束： <========================");
        //3.4、获取经销商分销业务员人员列表(分页)结束： <========================

//财务3.2
        String financeJson ="";
        try{
            financeJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.FINANCE,1+""));
        }catch (Exception e){
            log.error("获取经销商财务接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject fjson = JSONObject.parseObject(financeJson);
        Integer fcount = fjson.getInteger("count");
        JSONArray FINANCE = new JSONArray();
        if(fcount>100){
            int co = 0;
            if (fcount%100 != 0){
                co = (fcount/100) +2;
            }else{
                co = (fcount/100) +1;
            }
            for (int i = 1; i < co; i++) {
                try {
                    fjson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.FINANCE,i+"")));
                } catch (Exception e) {
                    log.error("循环获取经销商财务接口错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = fjson.getInteger("code");
                if(code==0){
                    FINANCE.addAll(fjson.getJSONArray("userlist"));
                }
            }
            success+=buildSysUser(FINANCE,loginNameSet,diyNameSet,2,failDetails,packageId,chainsMap);
            total+=FINANCE.size();
        }else{
            code = fjson.getInteger("code");
            if(code==0){
                FINANCE.addAll(fjson.getJSONArray("userlist"));
                success+=buildSysUser(FINANCE,loginNameSet,diyNameSet,2,failDetails,packageId,chainsMap);
                total+=FINANCE.size();
            }
        }

//3.3经销商仓管人员
        String warehouseJson ="";
        try{
            warehouseJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.WAREHOUSE,1+""));
        }catch (Exception e){
            log.error("获取经销商仓管接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject wjson = JSONObject.parseObject(warehouseJson);
        Integer wcount = wjson.getInteger("count");
        JSONArray WAREHOUSE = new JSONArray();
        if(wcount>100){
            int co = 0;
            if (wcount%100 != 0){
                co = (wcount/100) +2;
            }else{
                co = (wcount/100) +1;
            }
            for (int i = 1; i < co; i++) {
                try {
                    wjson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.WAREHOUSE,i+"")));
                } catch (Exception e) {
                    log.error("获取经销商仓管接口错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = wjson.getInteger("code");
                if(code==0){
                    WAREHOUSE.addAll(wjson.getJSONArray("userlist"));
                }
            }
            success+=buildSysUser(WAREHOUSE,loginNameSet,diyNameSet,3,failDetails,packageId,chainsMap);
            total+=WAREHOUSE.size();
        }else{
            code = wjson.getInteger("code");
            if(code==0){
                WAREHOUSE.addAll(wjson.getJSONArray("userlist"));
                success+=buildSysUser(WAREHOUSE,loginNameSet,diyNameSet,3,failDetails,packageId,chainsMap);
                total+=WAREHOUSE.size();
            }
        }

        //3.6经销商经销人员列表
        String staffListJson ="";
        JSONArray STAFF = new JSONArray();
        try {
            staffListJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.STAFF, 1+""));
        } catch (Exception e) {
            log.error("3.6经销商经销人员列表！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject json = JSONObject.parseObject(staffListJson);
        Integer Staffcount = json.getInteger("count");
        if(Staffcount>100){
            int co = 0;
            if (Staffcount%100 != 0){
                co = (Staffcount/100) +2;
            }else{
                co = (Staffcount/100) +1;
            }
            for (int i = 1; i < co; i++) {
                try {
                    json = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.STAFF,i+"")));
                } catch (Exception e) {
                    log.error("循环获取3.6接口错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = json.getInteger("code");
                if(code==0){
                    STAFF.addAll(json.getJSONArray("data"));
                }
            }
            success += buildSysUserStaff(STAFF, loginNameSet, diyNameSet, failDetails, packageId, chainsMap,8);
            total += STAFF.size();
        }else{
            code = json.getInteger("code");
            if(code==0){
                STAFF.addAll(json.getJSONArray("data"));
                success += buildSysUserStaff(STAFF, loginNameSet, diyNameSet, failDetails, packageId, chainsMap,8);
                total += STAFF.size();
            }
        }

        //3.7经销商倍增人员人员(门店经销商)
        String beizengJson ="";
        JSONArray bz = new JSONArray();
        try {
            beizengJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BZ, 1+""));
        } catch (Exception e) {
            log.error("3.7经销商倍增人员列表！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject bzJson = JSONObject.parseObject(beizengJson);
        Integer bzCount = bzJson.getInteger("count");
        if(bzCount>100){
            int co = 0;
            if (bzCount%100 != 0){
                co = (bzCount/100) +2;
            }else{
                co = (bzCount/100) +1;
            }
            for (int i = 1; i < co; i++) {
                try {
                    bzJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BZ,i+"")));
                } catch (Exception e) {
                    log.error("循环获取3.7经销商倍增人员错误！第"+i+"页");
                    customerAccessTokenUtil.reSetToken();
                    throw e;
                }
                code = bzJson.getInteger("code");
                if(code==0){
                    bz.addAll(bzJson.getJSONArray("data"));
                }
            }
            success += buildSysUserBz(bz, loginNameSet, diyNameSet, failDetails, packageId, chainsMap,5);
            total += bz.size();
        }else{
            code = bzJson.getInteger("code");
            if(code==0){
                bz.addAll(bzJson.getJSONArray("data"));
                success += buildSysUserBz(bz, loginNameSet, diyNameSet, failDetails, packageId, chainsMap,5);
                total += bz.size();
            }
        }




        //13.1 新零售经销商人员列表
        String xlsCustomerJson ="";
        try{
            xlsCustomerJson = HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.XLS,null));
        }catch (Exception e){
            log.error("获取新零售经销商人员接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        JSONObject xlsJson = JSONObject.parseObject(xlsCustomerJson);
        code = xlsJson.getInteger("code");
        if(code==0){
            JSONArray xlsMANAGER = new JSONArray();
            xlsMANAGER.addAll(xlsJson.getJSONArray("data"));
            success+=buildSysUser(xlsMANAGER,loginNameSet,diyNameSet,7,failDetails,packageId,chainsMap);
            total+=xlsMANAGER.size();
        }


        /*try {
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BOSSLIST, null)));
            code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 4, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫老板列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }*/

        /*try {
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.MANAGERLIST, null)));
            code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 5, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫经理列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }*/

        /*try {
            //获取厨卫业务员人员
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BUSINESSLIST, null)));
            code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 6, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫业务人员列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }*/

        try{
            syncLog.setTotalCount(total);
            syncLog.setSuccessCount(success);
            syncLog.setFaileCount((long)failDetails.size());
            syncLog.setEndTime(new Date());
            syncLog.setSyncType("diycustomer");

            if(failDetails.size()>0){
                syncFailDetailMapper.batchInsertSyncFailDetail(failDetails);
            }
        }catch (Exception e){
            log.error("解析用户插入解析日志出错",e);
            e.printStackTrace();
            throw new BusinessException("解析用户插入解析日志出错",e);
        }finally {
            syncLogService.insertSyncLog(syncLog);
        }


        return null;
    }


    /*
     * @description: 厨卫人员拉取
     * @author: qxm
     * @date: 2022/4/25 10:03
     */
  /*  public SyncLog analyCwCustomer() {
        if(deptId==null){
            deptId = Long.parseLong(configService.selectConfigByKey("libang.customer.deptid"));
        }
        if(roleIds==null){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.customer.roleid"));
            roleIds = new Long[]{roleId};
        }

        if (roleGuideIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.guide.roleid"));
            roleGuideIds = new Long[]{roleId};
        }
        if (roleCwIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.roleid"));
            roleCwIds = new Long[]{roleId};
        }
        if (roleCwDiyIds == null) {
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.roleid"));
            Long roleDiyId = Long.parseLong(configService.selectConfigByKey("libang.guide.roleid"));
            roleCwDiyIds = new Long[]{roleId, roleDiyId};
        }
        if(roleBusiness ==null ){
            Long roleId = Long.parseLong(configService.selectConfigByKey("libang.cw.business"));
            roleBusiness = new Long[]{roleId};
        }


        List<SyncFailDetail> failDetails = new ArrayList<>();
        SyncLog syncLog = new SyncLog();
        syncLog.setStartTime(new Date());
        String packageId = "diycustomer"+syncLog.getStartTime().getTime();
        syncLog.setPackageId(packageId);
        long total = 0;
        long success = 0;
        Set<String> loginNameSet = sysUserService.selectAllUserLoginName();
        Set<String> diyNameSet = sysUserService.selectDiyUserLoginName();

        Map<String,Customers> codeMap = customersMapper.selectCustomerMapByCode();

        Map<String,Customers> chainsMap = customersMapper.selectCustomerMapByChainsCode();

        try {
            // JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BOSSLIST, null)));
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.AllRole, null)));
            int code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, failDetails, packageId, chainsMap);
                //total += bossListJson.getJSONArray("userlist").size();
                total += bossListJson.getJSONArray("data").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

    *//*    try {
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.MANAGERLIST, null)));
            int code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 5, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫经理列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }

        try {
            //获取厨卫业务员人员
            JSONObject bossListJson = JSONObject.parseObject(HttpUtils.sendSSLGet(customerAccessTokenUtil.getUrl(UrlType.BUSINESSLIST, null)));
            int code = bossListJson.getInteger("code");
            if (code == 0) {
                success += buildSysUserEbr(bossListJson, loginNameSet, diyNameSet, 6, failDetails, packageId, chainsMap);
                total += bossListJson.getJSONArray("userlist").size();
            }
        } catch (Exception e) {
            log.error("获取厨卫业务人员列表接口错误！");
            customerAccessTokenUtil.reSetToken();
            throw e;
        }*//*

        try{
            syncLog.setTotalCount(total);
            syncLog.setSuccessCount(success);
            syncLog.setFaileCount((long)failDetails.size());
            syncLog.setEndTime(new Date());
            syncLog.setSyncType("diycustomer");

            if(failDetails.size()>0){
                syncFailDetailMapper.batchInsertSyncFailDetail(failDetails);
            }
        }catch (Exception e){
            log.error("解析用户插入解析日志出错",e);
            e.printStackTrace();
            throw new BusinessException("解析用户插入解析日志出错",e);
        }finally {
            syncLogService.insertSyncLog(syncLog);
        }


        return null;
    }*/

    /**
     * 构建 插入用户List
     * @param
     * @param loginNameSet 排重用
     * @param isManager
     * @return
     */
    public Integer buildSysUser(JSONArray jsonArray, Set<String> loginNameSet,Set<String> diyNameSet, int isManager, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap) {
        // JSONArray jsonArray = json.getJSONArray("userlist");
        int count = 0;
        List<String> interfaceList = new ArrayList<>();
        for (int x = 0; x < jsonArray.size(); x++) {
            JSONObject jsonObject = jsonArray.getJSONObject(x);
            String loginName = jsonObject.getString("userid");
            if (loginName != null && !"".equals(loginName)) {
                interfaceList.add(loginName);
            }
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jo = jsonArray.getJSONObject(i);

            try {
                String userNo = jo.getString("userNo");
                String enable = jo.getString("enable");
                String name = jo.getString("name");
                String status = jo.getString("status");
                Boolean isSync = jo.getBoolean("isSync");
                Boolean isDelete = jo.getBoolean("isDelete");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");
                String customerName = null;
                if (isManager != 0) {
                    customerName = jo.getString("depName");
                }
                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if (userid == null || userid.trim().equals("")) {
                    continue;
                }
                loginNameSet = sysUserService.selectAllUserLoginName();
                loginNameSet = loginNameSet.stream().map(String::toUpperCase).collect(Collectors.toSet());
                if (!loginNameSet.contains(userid.toUpperCase().trim())) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));
                    if (isManager == 0) {
                        user.setRoleIds(roleGuideIds);
                    }else if(isManager == 2){
                        user.setRoleIds(FinanceRoleIds);
                    }else if(isManager == 3){
                        user.setRoleIds(HouseRoleIds);
                    }else if(isManager == 7){
                        user.setRoleIds(roleXls);
                    }else{
                        user.setRoleIds(roleIds);
                    }
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);
                    userExt.setIdentity(UserIdentityUtil.UI_DIY);
                    userExt.setPostName("DIY经销商");
                    String orgCode = jo.getString("depCode");
                    String code = null;
                    if (cusMap.containsKey(orgCode)) {
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                        code = c.getChainsCode();
                    } else {
                        userExt.setOrgCode(orgCode);
                        userExt.setOrgName(customerName);
                    }
                    if (isManager == 0) {
                        userExt.setCustomerCode(userExt.getOrgCode());
                    } else {
                        userExt.setCustomerCode(jo.getString("depCode"));
                    }
//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                } else {
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/
                    //查询岗位现阶段存在的用户
                    SysUser user = sysUserService.selectUserByLoginName(userid);
                    if (("0").equals(user.getIsSync())) {
                        //查询该用户是否存在该接口返回中

                        if (user.getSysUserExt().getIsCustomerManager() == null) {
                            user.getSysUserExt().setIsCustomerManager(isManager + "");
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        /*判断不是导购角色，更新经销商角色*/
//                    if (isManager == 1 || isManager == 2 || isManager == 3) {
//                        user.setRoleIds(roleIds);
//                        sysUserService.updateUser(user);
//                    }
                        //根据用户id和角色id删除

                        if (isManager == 0) {
                            sysUserService.insertUserRole(user.getUserId(), roleGuideIds[0]);
                        }else if(isManager == 2){
                            sysUserService.insertUserRole(user.getUserId(), FinanceRoleIds[0]);
                        }else if(isManager == 3){
                            sysUserService.insertUserRole(user.getUserId(), HouseRoleIds[0]);
                        }else if (isManager ==7){
                            sysUserService.insertUserRole(user.getUserId(), roleXls[0]);
                        }else {
                            sysUserService.insertUserRole(user.getUserId(), roleIds[0]);
                        }
                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                                !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                                /*更新职位*/
                                user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                            }
                            if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                                /*更新经销商code*/
                                if (cusMap.containsKey(customerCode)) {
                                    Customers c = cusMap.get(customerCode);
                                    user.getSysUserExt().setOrgCode(c.getChainsCode());
                                    user.getSysUserExt().setOrgName(c.getChainsName());
                                } else {
                                    user.getSysUserExt().setOrgCode(customerCode);
                                    user.getSysUserExt().setOrgName(customerName);
                                }
                            }
                            log.info("updateSysUserExt" + customerCode);
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        if(("019").equals(user.getSysUserExt().getCustomerCode().substring(0, 3))){
                            user.getSysUserExt().setCustomerCode(user.getSysUserExt().getOrgCode());
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        if(user.getSysUserExt().getIsCustomerManager() !=null){
                            if(user.getSysUserExt().getIsCustomerManager().contains("0")){
                                if(("019").equals(user.getSysUserExt().getOrgCode().substring(0, 3))){
                                    if (cusMap.containsKey(customerCode)) {
                                        Customers c = cusMap.get(customerCode);
                                        user.getSysUserExt().setOrgCode(c.getChainsCode());
                                        user.getSysUserExt().setOrgName(c.getChainsName());
                                    } else {
                                        user.getSysUserExt().setOrgCode(customerCode);
                                        user.getSysUserExt().setOrgName(customerName);
                                    }
                                }
                            }
                        }
                        /*解决名称不一样的情况*/
                        if (!user.getUserName().equals(name)) {
                            user.setUserName(name);
                            sysUserService.updateUserInfo(user);
                        }
                        List<AssRelcustomerinfo> assRelcustomerinfoList = new ArrayList<>();
                        /*过滤导购*/
                        if (isManager != 0 && !("019").equals(user.getSysUserExt().getOrgCode().substring(0, 3)) && !("019").equals(user.getSysUserExt().getCustomerCode().substring(0, 3))) {
                            if (user.getSysUserExt().getCustomerCode() != null && !customerCode.equals(user.getSysUserExt().getCustomerCode())) {
                                assRelcustomerinfoList = assRelcustomerinfoService.selectRelcustomerinfoByKunnr(customerCode, user.getUserId().toString());
                                if (assRelcustomerinfoList != null && assRelcustomerinfoList.size() > 0) {
                                    assRelcustomerinfoService.deleteAssRelcustomerinfoByUserId(user.getUserId().toString(), customerCode);
                                }
                                assRelcustomerinfoList = assRelcustomerinfoService.selectRelcustomerinfoByKunnr(customerCode, user.getUserId().toString());
                                if (assRelcustomerinfoList == null || assRelcustomerinfoList.size() == 0) {
                                    AssRelcustomerinfo info = new AssRelcustomerinfo();
                                    info.setUserId(user.getUserId().toString());
                                    if (isManager == 0) {
                                        info.setMainkunnr(user.getSysUserExt().getOrgCode());
                                    } else {
                                        info.setMainkunnr(user.getSysUserExt().getCustomerCode());
                                    }

                                    if (cusMap.containsKey(info.getMainkunnr())) {
                                        Customers c = cusMap.get(info.getMainkunnr());
                                        info.setMainname1(c.getChainsName());
                                    }

//                            info.setMainname1(user.getSysUserExt().getOrgName());
                                    info.setKunnr(customerCode);
                                    if (customerName != null && customerName != "") {
                                        info.setName1(customerName);
                                    } else {
                                        if (isManager == 0) {
                                            String orgCode = jo.getString("depCode");
                                            String code = null;
                                            if (cusMap.containsKey(orgCode)) {
                                                Customers c = cusMap.get(orgCode);
                                                code = c.getChainsCode();
                                            } else {
                                                code = orgCode;
                                            }
                                            String Name = assRelcustomerinfoService.selectCustomerName(code);
                                            info.setName1(Name);
                                        } else {
                                            String Name = assRelcustomerinfoService.selectCustomerName(customerCode);
                                            info.setName1(Name);
                                        }
                                    }
                                    info.setGlfs("用户关联");
                                    assRelcustomerinfoService.insertAssRelcustomerinfo(info);

                                }
                            }
                        }
                    }
                }
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }
        List<String> loginNames = sysUserService.selectUserByIsManager(isManager);
        List<String> collect =new ArrayList<>();
        loginNames.removeAll(interfaceList);
        collect.addAll(loginNames);
        if (collect.size() > 0) {
            for (String login : collect
            ) {
                SysUser u = sysUserService.selectUserByLoginName(login);
                if (("0").equals(u.getIsSync())) {
                    String CustomerManager = u.getSysUserExt().getIsCustomerManager();
                    String[] manager = CustomerManager.split(",");
                    if (manager.length > 1) {
                        List<String> resultList = new ArrayList<>(manager.length);
                        Collections.addAll(resultList,manager);
                        Object i=isManager;
                        resultList.remove(i.toString());
                        String IsManager = resultList.stream().map(String::valueOf).collect(Collectors.joining(","));
                        u.getSysUserExt().setIsCustomerManager(IsManager);
                        sysUserExtMapper.updateSysUserExt(u.getSysUserExt());
                    } else {
                        u.setQuit("2");
                        sysUserService.updateUserInfo(u);
                    }
                }
            }
        }
        return count;
    }


    /**
     * 构建 厨卫插入用户List
     * @param json
     * @param loginNameSet 排重用
     * @param
     * @return
     */
    public Integer buildSysUserEbr(JSONObject json, Set<String> loginNameSet,Set<String> diyNameSet, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap){
        JSONArray jsonArray = json.getJSONArray("data");
        int count = 0;
        List<String > interfaceList4=new ArrayList<>();
        List<String > interfaceList5=new ArrayList<>();
        List<String > interfaceList7=new ArrayList<>();
        List<String > interfaceList9=new ArrayList<>();
        for(int x =0;x<jsonArray.size();x++){
            JSONObject jsonObject = jsonArray.getJSONObject(x);
            String loginName = jsonObject.getString("userid");
            String role=jsonObject.getString("role_code");
            if(loginName !=null && !"".equals(loginName)){
                if(role.equals("boss")){
                    interfaceList4.add(loginName);
                }else if(role.equals("manager")){
                    interfaceList5.add(loginName);
                }else if(role.equals("CWWH")){
                    interfaceList7.add(loginName);
                }else if(role.equals("business")){
                    interfaceList9.add(loginName);
                }

            }
        }
        int isManager=1;
        for(int i =0;i<jsonArray.size();i++){
            JSONObject jo = jsonArray.getJSONObject(i);

            try {
                String role=jo.getString("role_code");
              /*  if(role.equals("business")){
                    continue;
                }*/
                String userNo = jo.getString("userNo");
                String name = jo.getString("name");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");
                if(role.equals("boss")){
                    isManager=4;
                }else if(role.equals("manager")){
                    isManager=5;
                }else if(role.equals("CWWH")){
                    isManager=7;
                }else if(role.equals("business")){
                    isManager=9;
                }


                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if(userid==null||userid.trim().equals("")){
                    continue;
                }
                loginNameSet = sysUserService.selectAllUserLoginName();
                diyNameSet = sysUserService.selectDiyUserLoginName();
                loginNameSet = loginNameSet.stream().map(String::toUpperCase).collect(Collectors.toSet());
                if (!loginNameSet.contains(userid.toUpperCase().trim())) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));
                    if(role.equals("business")){
                        user.setRoleIds(roleBusiness);
                    } else if (diyNameSet.contains(userid) && !role.equals("business")) {
                        user.setRoleIds(roleCwDiyIds);
                    } else if(!role.equals("business")){
                        user.setRoleIds(roleCwIds);
                    }
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");

                    userExt.setIdentity(UserIdentityUtil.UI_CW);// 1=diy,2=ebr,3=all

                    userExt.setCustomerCode(jo.getString("depCode"));
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);

                    userExt.setPostName("CW经销商");
                    String orgCode = jo.getString("depCode");
                    if(cusMap.containsKey(orgCode)){
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                    }else{
                        userExt.setOrgCode(orgCode);
                        //userExt.setOrgName(customerName);
                    }

//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                }else{
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/
                    SysUser user = sysUserService.selectUserByLoginName(userid);
                    if(("0").equals(user.getIsSync())){
                        //每个接口查询俩次是否有重复开始查询一次，快结束查一次
                        //查询该用户是否存在该接口返回中
                        if(user.getSysUserExt().getIsCustomerManager()==null ){
                            user.getSysUserExt().setIsCustomerManager(isManager+ "");
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        List<AssRelcustomerinfo>  assRelcustomerinfoList=new ArrayList<>();
                        if ( !("019").equals(user.getSysUserExt().getOrgCode().substring(0,3)) && !("019").equals(user.getSysUserExt().getCustomerCode().substring(0,3))) {
                            if (!customerCode.equals(user.getSysUserExt().getCustomerCode())) {
                                assRelcustomerinfoList = assRelcustomerinfoService.selectRelcustomerinfoByKunnr(customerCode, user.getUserId().toString());
                                if (assRelcustomerinfoList != null && assRelcustomerinfoList.size() > 0) {
                                    assRelcustomerinfoService.deleteAssRelcustomerinfoByUserId(user.getUserId().toString(), customerCode);
                                }
                                assRelcustomerinfoList = assRelcustomerinfoService.selectRelcustomerinfoByKunnr(customerCode, user.getUserId().toString());
                                if (assRelcustomerinfoList == null || assRelcustomerinfoList.size() == 0) {
                                    String customerName = assRelcustomerinfoService.selectCustomerName(customerCode);
                                    AssRelcustomerinfo info = new AssRelcustomerinfo();
                                    info.setUserId(user.getUserId().toString());
                                    info.setMainkunnr(user.getSysUserExt().getCustomerCode());
                                    info.setMainname1(user.getSysUserExt().getOrgName());
                                    info.setKunnr(customerCode);
                                    if (customerName != null && customerName != "") {
                                        info.setName1(customerName);
                                    }else {
                                        if (cusMap.containsKey(info.getKunnr())) {
                                            Customers c = cusMap.get(info.getKunnr());
                                            info.setName1(c.getChainsName());
                                        }
                                    }
                                    info.setGlfs("用户关联");
                                    assRelcustomerinfoService.insertAssRelcustomerinfo(info);

                                }

                            }
                        }
                        if (diyNameSet.contains(userid) && !role.equals("business")) {
                            sysUserService.insertUserRole(user.getUserId(), roleIds[0]);
                            sysUserService.insertUserRole(user.getUserId(), roleCwIds[0]);
                        } else if(role.equals("business")){
                            sysUserService.insertUserRole(user.getUserId(), roleBusiness[0]);
                        }else if(!role.equals("business")) {
                            sysUserService.insertUserRole(user.getUserId(), roleCwIds[0]);
                        }

                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                                !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                                /*更新职位*/
                                user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                            }
                            if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                                /*更新经销商code*/
                                if (cusMap.containsKey(customerCode)) {
                                    Customers c = cusMap.get(customerCode);
                                    user.getSysUserExt().setOrgCode(c.getChainsCode());
                                    user.getSysUserExt().setOrgName(c.getChainsName());
                                } else {
                                    user.getSysUserExt().setOrgCode(customerCode);

                                }
                            }
                            log.info("updateSysUserExt" + customerCode);
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        /*解决名称不一样的情况*/
                        if(!user.getUserName().equals(name)){
                            user.setUserName(name);
                            sysUserService.updateUserInfo(user);
                        }
                        //更新 identity 1=diy,2=ebr,3=all
                        if (diyNameSet.contains(userid) && !"1,2".equals(user.getSysUserExt().getIdentity())) {

                            log.info("updateSysUserExt identity 1,2" + userid);
                            user.getSysUserExt().setIdentity(UserIdentityUtil.UI_DIY+","+UserIdentityUtil.UI_CW);

                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        if (!diyNameSet.contains(userid) && !"2".equals(user.getSysUserExt().getIdentity())) {
                            log.info("updateSysUserExt identity 2" + userid);
                            user.getSysUserExt().setIdentity(UserIdentityUtil.UI_CW);
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }
        //厨卫老板
        List<String> loginNames4=sysUserService.selectUserByIsManager(4);
        //厨卫经理
        List<String> loginNames5=sysUserService.selectUserByIsManager(5);
        //厨卫仓管
        List<String> loginNames7=sysUserService.selectUserByIsManager(7);
        //厨卫业务人员
        List<String> loginNames9=sysUserService.selectUserByIsManager(9);
        List<String> collect4 =new ArrayList<>();
        loginNames4.removeAll(interfaceList4);
        collect4.addAll(loginNames4);
        this.uninoncw(collect4, 4);
        List<String> collect5 =new ArrayList<>();
        loginNames5.removeAll(interfaceList5);
        collect5.addAll(loginNames5);
        this.uninoncw(collect5, 5);
        List<String> collect7 =new ArrayList<>();
        loginNames7.removeAll(interfaceList7);
        collect7.addAll(loginNames7);
        this.uninoncw(collect7, 7);
        List<String> collect9 =new ArrayList<>();
        loginNames9.removeAll(interfaceList9);
        collect9.addAll(loginNames9);
        this.uninoncw(collect9, 9);
        return count;
    }

    void uninoncw( List<String> collect, int isManager){
        if (collect.size() > 0) {
            for (String login:collect
            ) {
                SysUser u = sysUserService.selectUserByLoginName(login);
                if(("0").equals(u.getIsSync())){
                    String CustomerManager= u.getSysUserExt().getIsCustomerManager();
                    String[] manager=CustomerManager.split(",");
                    if(manager.length >1){
                        List<String> resultList = new ArrayList<>(manager.length);
                        Collections.addAll(resultList,manager);
                        Object i=isManager;
                        resultList.remove(i.toString());
                        String IsManager = resultList.stream().map(String::valueOf).collect(Collectors.joining(","));
                        u.getSysUserExt().setIsCustomerManager(IsManager);
                        sysUserExtMapper.updateSysUserExt(u.getSysUserExt());
                    }else{
                        u.setQuit("2");
                        sysUserService.updateUserInfo(u);
                    }
                }
            }
        }
    }

    /***
     *
     * 3.6
     * **/
    public Integer buildSysUserStaff(JSONArray STAFF, Set<String> loginNameSet,Set<String> diyNameSet, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap,Integer isManager){
        //JSONArray jsonArray = json.getJSONArray("data");
        int count = 0;
        List<String > interfaceList8=new ArrayList<>();
        for(int x =0;x<STAFF.size();x++){
            JSONObject jsonObject = STAFF.getJSONObject(x);
            String loginName = jsonObject.getString("userid");
            if(loginName !=null && !"".equals(loginName)){
                    interfaceList8.add(loginName);
            }
        }
        for(int i =0;i<STAFF.size();i++){
            JSONObject jo = STAFF.getJSONObject(i);

            try {
                String userNo = jo.getString("userNo");
                String name = jo.getString("name");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");


                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if(userid==null||userid.trim().equals("")){
                    continue;
                }
                loginNameSet = sysUserService.selectAllUserLoginName();
                diyNameSet = sysUserService.selectDiyUserLoginName();
                loginNameSet = loginNameSet.stream().map(String::toUpperCase).collect(Collectors.toSet());
                if (!loginNameSet.contains(userid.toUpperCase().trim())) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));
                    user.setRoleIds(roleStoreIds);
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");

                    userExt.setIdentity(UserIdentityUtil.UI_DIY);// 1=diy,2=ebr,3=all

                    userExt.setCustomerCode(jo.getString("depCode"));
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);

                    userExt.setPostName("DIY经销商");
                    String orgCode = jo.getString("depCode");
                    if(cusMap.containsKey(orgCode)){
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                    }else{
                        userExt.setOrgCode(orgCode);
                    }

//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                    diyNameSet.add(userid);
                }else{
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/
                    SysUser user = sysUserService.selectUserByLoginName(userid);
                    if(("0").equals(user.getIsSync())){
                        //每个接口查询俩次是否有重复开始查询一次，快结束查一次
                        //查询该用户是否存在该接口返回中
                        sysUserService.insertUserRole(user.getUserId(), roleStoreIds[0]);
                        if (user.getSysUserExt().getIsCustomerManager() == null) {
                            user.getSysUserExt().setIsCustomerManager(isManager + "");
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                                !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                                /*更新职位*/
                                user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                            }
                            if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                                /*更新经销商code*/
                                if (cusMap.containsKey(customerCode)) {
                                    Customers c = cusMap.get(customerCode);
                                    user.getSysUserExt().setOrgCode(c.getChainsCode());
                                    user.getSysUserExt().setOrgName(c.getChainsName());
                                } else {
                                    user.getSysUserExt().setOrgCode(customerCode);
                                }
                            }
                            log.info("updateSysUserExt" + customerCode);
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        /*解决名称不一样的情况*/
                        if (!user.getUserName().equals(name)) {
                            user.setUserName(name);
                            sysUserService.updateUserInfo(user);
                        }


                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }
        List<String> loginNames=sysUserService.selectUserByIsManager(isManager);
        List<String> collect =new ArrayList<>();
        loginNames.removeAll(interfaceList8);
        collect.addAll(loginNames);
        if (collect.size() > 0) {
            for (String login:collect
            ) {
                SysUser u = sysUserService.selectUserByLoginName(login);
                if(("0").equals(u.getIsSync())){
                    String CustomerManager= u.getSysUserExt().getIsCustomerManager();
                    String[] manager=CustomerManager.split(",");
                    if(manager.length >1){
                        List<String> resultList = new ArrayList<>(manager.length);
                        Collections.addAll(resultList,manager);
                        Object i=isManager;
                        resultList.remove(i.toString());
                        String IsManager = resultList.stream().map(String::valueOf).collect(Collectors.joining(","));
                        u.getSysUserExt().setIsCustomerManager(IsManager);
                        sysUserExtMapper.updateSysUserExt(u.getSysUserExt());
                    }else{
                        u.setQuit("2");
                        sysUserService.updateUserInfo(u);
                    }
                }
            }
        }

        return count;
    }

    /***
     *
     *3.7经销商经销人员门店
     * **/
    public Integer buildSysUserBz(JSONArray BZ, Set<String> loginNameSet,Set<String> diyNameSet, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap,Integer isManager){
        //JSONArray jsonArray = json.getJSONArray("data");
        int count = 0;
        List<String > interfaceList5=new ArrayList<>();
        for(int x =0;x<BZ.size();x++){
            JSONObject jsonObject = BZ.getJSONObject(x);
            String loginName = jsonObject.getString("userid");
            if(loginName !=null && !"".equals(loginName)){
                    interfaceList5.add(loginName);
            }
        }
        for(int i =0;i<BZ.size();i++){
            JSONObject jo = BZ.getJSONObject(i);

            try {
                String userNo = jo.getString("userNo");
                String name = jo.getString("name");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");


                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if(userid==null||userid.trim().equals("")){
                    continue;
                }
                loginNameSet = sysUserService.selectAllUserLoginName();
                diyNameSet = sysUserService.selectDiyUserLoginName();
                loginNameSet = loginNameSet.stream().map(String::toUpperCase).collect(Collectors.toSet());
                if (!loginNameSet.contains(userid.toUpperCase().trim())) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));
                    user.setRoleIds(roleStoreIds);
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");

                    userExt.setIdentity(UserIdentityUtil.UI_DIY);// 1=diy,2=ebr,3=all

                    userExt.setCustomerCode(jo.getString("depCode"));
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);

                    userExt.setPostName("DIY经销商");
                    String orgCode = jo.getString("depCode");
                    if(cusMap.containsKey(orgCode)){
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                    }else{
                        userExt.setOrgCode(orgCode);
                    }

//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                    diyNameSet.add(userid);
                }else{
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/
                    SysUser user = sysUserService.selectUserByLoginName(userid);
                    if(("0").equals(user.getIsSync())){
                        //每个接口查询俩次是否有重复开始查询一次，快结束查一次
                        //查询该用户是否存在该接口返回中
                        sysUserService.insertUserRole(user.getUserId(), roleStoreIds[0]);
                        if (user.getSysUserExt().getIsCustomerManager() == null) {
                            user.getSysUserExt().setIsCustomerManager(isManager + "");
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                                !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                                /*更新职位*/
                                user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                            }
                            if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                                /*更新经销商code*/
                                if (cusMap.containsKey(customerCode)) {
                                    Customers c = cusMap.get(customerCode);
                                    user.getSysUserExt().setOrgCode(c.getChainsCode());
                                    user.getSysUserExt().setOrgName(c.getChainsName());
                                } else {
                                    user.getSysUserExt().setOrgCode(customerCode);
                                }
                            }
                            log.info("updateSysUserExt" + customerCode);
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        /*解决名称不一样的情况*/
                        if (!user.getUserName().equals(name)) {
                            user.setUserName(name);
                            sysUserService.updateUserInfo(user);
                        }


                    }
                }
            }catch (Exception e){
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }
        List<String> loginNames=sysUserService.selectUserByIsManager(isManager);
        List<String> collect =new ArrayList<>();
        loginNames.removeAll(interfaceList5);
        collect.addAll(loginNames);
        if (collect.size() > 0) {
            for (String login:collect
            ) {
                SysUser u = sysUserService.selectUserByLoginName(login);
                if(("0").equals(u.getIsSync())){
                    String CustomerManager= u.getSysUserExt().getIsCustomerManager();
                    String[] manager=CustomerManager.split(",");
                    if(manager.length >1){
                        List<String> resultList = new ArrayList<>(manager.length);
                        Collections.addAll(resultList,manager);
                        Object i=isManager;
                        resultList.remove(i.toString());
                        String IsManager = resultList.stream().map(String::valueOf).collect(Collectors.joining(","));
                        u.getSysUserExt().setIsCustomerManager(IsManager);
                        sysUserExtMapper.updateSysUserExt(u.getSysUserExt());
                    }else{
                        u.setQuit("2");
                        sysUserService.updateUserInfo(u);
                    }
                }
            }
        }

        return count;
    }
    /**
     * 构建 插入分销 3.4 分销业务员人员 List
     * @param
     * @param loginNameSet 排重用
     * @param isManager
     * @return
     */
    public Integer buildSysUserFx(JSONArray jsonArray, Set<String> loginNameSet, int isManager, List<SyncFailDetail> failDetails,String packageId, Map<String,Customers> cusMap){
        //JSONArray jsonArray = json.getJSONArray("userlist");
        int count = 0;
        List<String > interfaceList=new ArrayList<>();
        for(int x =0;x<jsonArray.size();x++){
            JSONObject jsonObject = jsonArray.getJSONObject(x);
            String loginName = jsonObject.getString("userid");
            if(loginName!=null && !"".equals(loginName)){
                interfaceList.add(loginName);
            }
        }
        for(int i =0;i<jsonArray.size();i++){
            JSONObject jo = jsonArray.getJSONObject(i);

            try {
                String userNo = jo.getString("userNo");
                String enable = jo.getString("enable");
                String name = jo.getString("name");
                String status = jo.getString("status");
                Boolean isSync = jo.getBoolean("isSync");
                Boolean isDelete = jo.getBoolean("isDelete");
                String userid = jo.getString("userid");
                String customerCode = jo.getString("depCode");
                String customerName=null;
                if (isManager != 0) {
                    customerName=jo.getString("depName");
                }
                /*企业微信中成员启用状态。1表示启用的成员，0表示被禁用*/
                /*if(enable==null||enable.trim().equals("0")){
                    continue;
                }
                *//*企业微信中人员激活状态: 1=已激活，2=已禁用，4=未激活，5=退出企业。*//*
                if(status==null || !status.trim().equals("1")){
                    continue;
                }
                *//*人员信息是否已更新入企业微信。true：更新成功false：更新失败*//*
                if(!isSync){
                    continue;
                }
                *//*人员信息是否被标记删除true：删除，false：未删除*//*
                if(isDelete){
                    continue;
                }*/
                if(userid==null||userid.trim().equals("")){
                    continue;
                }
                loginNameSet = sysUserService.selectAllUserLoginName();
                loginNameSet = loginNameSet.stream().map(String::toUpperCase).collect(Collectors.toSet());
                if (!loginNameSet.contains(userid.toUpperCase().trim())) {

                    /** =============用户基本信息构建 start===============*/
                    SysUser user = new SysUser();
                    user.setCompanyId(1L);
                    user.setDeptId(deptId);
                    user.setLoginName(userid);
                    user.setUserName(jo.getString("name"));
                    user.setPhonenumber(jo.getString("mobile"));

                    if (isManager == 6) {
                        user.setRoleIds(roleFXDiyIds);
                    } else {
                        user.setRoleIds(roleIds);
                    }
                    user.setPassword(user.getLoginName());
                    user.setSalt(ShiroSaltUtil.randomSalt());
                    user.setPassword(EncryptPassWordClass.encryptPassword(user.getLoginName(), user.getPassword(), user.getSalt()));


                    /** =============用户基本信息构建 end===============*/


                    /**======== 构建diy用户扩展信息表 start===============*/
                    SysUserExt userExt = new SysUserExt();
                    userExt.setSalesLevel("customer_level");
                    userExt.setCustomerCode(jo.getString("depCode"));
                    userExt.setIsCustomerManager(isManager + "");
                    userExt.setSapEmployeeId(userNo);

                    userExt.setPostName("经销商分销业务员");
                    String orgCode = jo.getString("depCode");
                    if(cusMap.containsKey(orgCode)){
                        Customers c = cusMap.get(orgCode);
                        userExt.setOrgCode(c.getChainsCode());
                        userExt.setOrgName(c.getChainsName());
                    }else{
                        userExt.setOrgCode(orgCode);
                    }

//                    /**======== 构建diy用户wx信息表 start===============*/
//                    WxQyUser wxQyUser = new WxQyUser();
//                    wxQyUser.setUserid(jo.getString("userid"));
//                    wxQyUser.setMobile(jo.getString("mobile"));
                    user.setSysUserExt(userExt);
                    insertUserAndExt(user);
                    count++;
                    loginNameSet.add(userid);
                }else{
                    /*已经存在的，需要先清空职位，更新职位
                    （首先查询当前用户职位，判断当前职位是否包含当前职位，不存在就更新，存在就不更新）*/

                    SysUser user = sysUserService.selectUserByLoginName(userid);
                    user.getSysUserExt().setSalesLevel("customer_level");
                    if(("0").equals(user.getIsSync())){
                        //每个接口查询俩次是否有重复开始查询一次，快结束查一次
                        //查询该用户是否存在该接口返回中
                        if(user.getSysUserExt().getIsCustomerManager()==null ){
                            user.getSysUserExt().setIsCustomerManager(isManager+ "");
                            sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        }
                        /*判断不是导购角色，更新经销商角色*/
//                    if (isManager == 1 || isManager == 2 || isManager == 3) {
//                        user.setRoleIds(roleIds);
//                        sysUserService.updateUser(user);
//                    }

                        sysUserService.insertUserRole(user.getUserId(), roleFXDiyIds[0]);

                        if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "") ||
                                !user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                            if (!user.getSysUserExt().getIsCustomerManager().contains(isManager + "")) {
                                /*更新职位*/
                                user.getSysUserExt().setIsCustomerManager(user.getSysUserExt().getIsCustomerManager() + "," + isManager);
                            }
                            if (!user.getSysUserExt().getCustomerCode().equals(customerCode) || !user.getSysUserExt().getOrgCode().equals(customerCode)) {
                                /*更新经销商code*/
                                if (cusMap.containsKey(customerCode)) {
                                    Customers c = cusMap.get(customerCode);
                                    user.getSysUserExt().setOrgCode(c.getChainsCode());
                                    user.getSysUserExt().setOrgName(c.getChainsName());
                                } else {
                                    user.getSysUserExt().setOrgCode(customerCode);
                                }
                            }
                            log.info("updateSysUserExt" + customerCode);
                        }
                        sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                        /*解决名称不一样的情况*/
                        if(!user.getUserName().equals(name)) {
                            user.setUserName(name);
                            sysUserService.updateUserInfo(user);
                        }
                    }
                    sysUserExtMapper.updateSysUserExt(user.getSysUserExt());
                }
            }catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
                SyncFailDetail syncFailDetail = new SyncFailDetail();
                syncFailDetail.setPackageId(packageId);
                syncFailDetail.setFailReason(e.getMessage());
                syncFailDetail.setDataJson(jo.toJSONString());
                syncFailDetail.setFailLevel("1");
                syncFailDetail.setExceptionType(e.getClass().getSimpleName());
                failDetails.add(syncFailDetail);
            }
        }
        List<String> loginNames = sysUserService.selectUserByIsManager(isManager);
        List<String> collect =new ArrayList<>();
        loginNames.removeAll(interfaceList);
        collect.addAll(loginNames);
        if (collect.size() > 0) {
            for (String login : collect
            ) {
                SysUser u = sysUserService.selectUserByLoginName(login);
                if (("0").equals(u.getIsSync())) {
                    String CustomerManager = u.getSysUserExt().getIsCustomerManager();
                    String[] manager = CustomerManager.split(",");
                    if (manager.length > 1) {
                        List<String> resultList = new ArrayList<>(manager.length);
                        Collections.addAll(resultList,manager);
                        Object i=isManager;
                        resultList.remove(i.toString());
                        String IsManager = resultList.stream().map(String::valueOf).collect(Collectors.joining(","));
                        u.getSysUserExt().setIsCustomerManager(IsManager);
                        sysUserExtMapper.updateSysUserExt(u.getSysUserExt());
                    } else {
                        u.setQuit("2");
                        sysUserService.updateUserInfo(u);
                    }
                }
            }
        }
        return count;
    }


    @Transactional(rollbackFor = Exception.class)
    public Integer insertUserAndExt(SysUser user) {
        // 新增用户信息
        int rows = sysUserMapper.insertUser(user);

        // 新增用户与角色管理
        sysUserService.insertUserRole(user);
        SysUserExt userExt = user.getSysUserExt();
        userExt.setUserId(user.getUserId());

        int i1 = sysUserExtMapper.insertOrUpSysUserExt(userExt);


        //wxQyUser.setSysUserId(user.getUserId());
//        int i2 =wxQyUserMapper.insertWxQyUser(wxQyUser);
//        if(i1+i2<2){
//            throw new RuntimeException("插入UserExt或WxUser错误");
//        }
        return rows;
    }
}
